@@ -60,9 +60,13 @@ static HTMLClueClass *parent_class = NULL;
 #define HCF_CLASS(x) HTML_CLUEFLOW_CLASS (HTML_OBJECT (x)->klass)
 #define HTML_IS_PLAIN_PAINTER(obj)              (GTK_CHECK_TYPE ((obj), HTML_TYPE_PLAIN_PAINTER))
 
-inline HTMLHAlignType html_clueflow_get_halignment (HTMLClueFlow *flow);
-static gchar * get_item_number_str (HTMLClueFlow *flow);
-static void update_items_after_indentation_change (HTMLClueFlow *flow);
+inline HTMLHAlignType html_clueflow_get_halignment          (HTMLClueFlow *flow);
+static gchar *        get_item_number_str                   (HTMLClueFlow *flow);
+static guint          get_post_padding                      (HTMLClueFlow *flow, 
+							     guint pad);
+static int            get_similar_depth                     (HTMLClueFlow *self, 
+							     HTMLClueFlow *neighbor);
+
 
 #define CLUEFLOW_INDENT             "        "
 #define CLUEFLOW_BLOCKQUOTE_CITE    "> "
@@ -94,10 +98,22 @@ is_levels_equal (HTMLClueFlow *me, HTMLClueFlow *you)
 	return !memcmp (me->levels->data, you->levels->data, you->levels->len);
 }
 
+static inline gboolean
+is_item (HTMLClueFlow *flow)
+{
+	return flow && flow->style == HTML_CLUEFLOW_STYLE_LIST_ITEM;
+}
+
 static void
 destroy (HTMLObject *self)
 {
-	g_byte_array_free (HTML_CLUEFLOW (self)->levels, TRUE);
+	HTMLClueFlow *flow = HTML_CLUEFLOW (self);
+
+	g_byte_array_free (flow->levels, TRUE);
+	if (flow->item_color) {
+		html_color_unref (flow->item_color);
+		flow->item_color = NULL;
+	}
 
 	(* HTML_OBJECT_CLASS (parent_class)->destroy) (self);
 }
@@ -113,12 +129,10 @@ copy (HTMLObject *self,
 	HTML_CLUEFLOW (dest)->item_type = HTML_CLUEFLOW (self)->item_type;
 	HTML_CLUEFLOW (dest)->item_number = HTML_CLUEFLOW (self)->item_number;
 	HTML_CLUEFLOW (dest)->clear = HTML_CLUEFLOW (self)->clear;
-}
+	HTML_CLUEFLOW (dest)->item_color = HTML_CLUEFLOW (self)->item_color;
 
-static inline gboolean
-is_item (HTMLClueFlow *flow)
-{
-	return flow && flow->style == HTML_CLUEFLOW_STYLE_LIST_ITEM;
+	if (HTML_CLUEFLOW (dest)->item_color)
+		html_color_ref (HTML_CLUEFLOW (dest)->item_color);
 }
 
 static inline gboolean
@@ -158,7 +172,9 @@ get_prev_relative_item (HTMLObject *self)
 	prev = self->prev;
 	while (prev 
 	       && HTML_IS_CLUEFLOW (prev) 
-	       && HTML_CLUEFLOW (prev)->levels->len > HTML_CLUEFLOW (self)->levels->len
+	       && (HTML_CLUEFLOW (prev)->levels->len > HTML_CLUEFLOW (self)->levels->len
+		   || (HTML_CLUEFLOW (prev)->levels->len == HTML_CLUEFLOW (self)->levels->len
+		       && !is_item (HTML_CLUEFLOW (prev))))
 	       && !memcmp (HTML_CLUEFLOW (prev)->levels->data,
 			   HTML_CLUEFLOW (self)->levels->data,
 			   HTML_CLUEFLOW (self)->levels->len))
@@ -174,8 +190,11 @@ get_next_relative_item (HTMLObject *self)
 	HTMLObject *next;
 
 	next = self->next;
-	while (next && HTML_IS_CLUEFLOW (next)
-	       && HTML_CLUEFLOW (next)->levels->len > HTML_CLUEFLOW (self)->levels->len
+	while (next 
+	       && HTML_IS_CLUEFLOW (next)
+	       && (HTML_CLUEFLOW (next)->levels->len > HTML_CLUEFLOW (self)->levels->len
+		   || (HTML_CLUEFLOW (next)->levels->len == HTML_CLUEFLOW (self)->levels->len
+		       && !is_item (HTML_CLUEFLOW (next))))
 	       && !memcmp (HTML_CLUEFLOW (next)->levels->data,
 			   HTML_CLUEFLOW (self)->levels->data, 
 			   HTML_CLUEFLOW (self)->levels->len))
@@ -186,22 +205,26 @@ get_next_relative_item (HTMLObject *self)
 }
 
 static void
-update_item_number (HTMLObject *self)
+update_item_number (HTMLObject *self, HTMLEngine *e)
 {
 	HTMLObject *prev, *next;
 
 	if (!self || !is_item (HTML_CLUEFLOW (self)))
 		return;
 
+	/* printf ("update_item_number\n"); */
+
 	prev = get_prev_relative_item (self);
 	if (items_are_relative (prev, self))
 		HTML_CLUEFLOW (self)->item_number = HTML_CLUEFLOW (prev)->item_number + 1;
 	else if (is_item (HTML_CLUEFLOW (self)))
 		HTML_CLUEFLOW (self)->item_number = 1;
+	html_engine_queue_draw (e, self);
 
 	next = self;
 	while ((next = get_next_relative_item (next)) && items_are_relative (self, next)) {
 		HTML_CLUEFLOW (next)->item_number = HTML_CLUEFLOW (self)->item_number + 1;
+		html_engine_queue_draw (e, next);
 		self = next;
 	}
 }
@@ -229,13 +252,13 @@ op_helper (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left,
 		   e->cursor->position --; */
 	}
 	if (cut)
-		html_clueflow_remove_text_slaves (HTML_CLUEFLOW (self));
+		html_clue_remove_text_slaves (HTML_CLUE (self));
 	o = cut
 		? (*HTML_OBJECT_CLASS (parent_class)->op_cut) (self, e, from, to, left, right, len)
 		: (*HTML_OBJECT_CLASS (parent_class)->op_copy) (self, NULL, e, from, to, len);
 
 	if (!cut && o) {
-		html_clueflow_remove_text_slaves (HTML_CLUEFLOW (o));
+		html_clue_remove_text_slaves (HTML_CLUE (o));
 	}
 
 	return o;
@@ -257,15 +280,15 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 
 	rv = op_helper (self, e, from, to, left, right, len, TRUE);
 
-	if (prev) {
-		update_item_number (prev);
+	if (prev && from) {
+		update_item_number (prev, e);
 		if (prev->next == self)
-			update_item_number (self);
+			update_item_number (self, e);
 	}
-	if (next) {
+	if (next && to) {
 		if (next->prev == self)
-			update_item_number (self);
-		update_item_number (next);
+			update_item_number (self, e);
+		update_item_number (next, e);
 	}
 
 	return rv;
@@ -301,34 +324,31 @@ static void
 split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, gint level, GList **left, GList **right)
 {
 	set_around_size (child);
-	html_clueflow_remove_text_slaves (HTML_CLUEFLOW (self));
+	html_clue_remove_text_slaves (HTML_CLUE (self));
 	(*HTML_OBJECT_CLASS (parent_class)->split) (self, e, child, offset, level, left, right);
 
-	update_item_number (self);
+	update_item_number (self, e);
 }
 
 static gboolean
 merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **right, HTMLCursor *cursor)
 {
 	HTMLClueFlow *cf1, *cf2;
+	HTMLObject *cf2_next_relative;
 	gboolean rv;
 
 	cf1 = HTML_CLUEFLOW (self);
 	cf2 = HTML_CLUEFLOW (with);
 
-	html_clueflow_remove_text_slaves (cf1);
-	html_clueflow_remove_text_slaves (cf2);
+	html_clue_remove_text_slaves (HTML_CLUE (cf1));
+	html_clue_remove_text_slaves (HTML_CLUE (cf2));
+
+	cf2_next_relative = get_next_relative_item (with);
 
 	set_tail_size (self);
 	set_head_size (with);
 
-	/* printf ("merge flows\n"); */
-
 	if (html_clueflow_is_empty (cf1)) {
-		cf1->style = cf2->style;
-		copy_levels (cf1->levels, cf2->levels);
-		cf1->item_type = cf2->item_type;
-		cf1->item_number = cf2->item_number - 1;
 		self->x = with->x;
 		self->y = with->y;
 		self->width = with->width;
@@ -341,11 +361,27 @@ merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **
 
 	rv = (* HTML_OBJECT_CLASS (parent_class)->merge) (self, with, e, left, right, cursor);
 
-	update_item_number (self);
-	cf1->item_number --;
-	update_item_number (with);
-	cf1->item_number ++;
+	if (rv) {
+		if (is_item (cf1)) {
+			/* cf2 will be removed, update item numbers around
+			   as if it was already removed - it has to have
+			   the same item style as cf1 to not break item lists*/
+			g_byte_array_free (cf2->levels, TRUE);
+			cf2->levels = html_clueflow_dup_levels (cf1);
+			cf2->style = cf1->style;
+			cf2->item_type = cf1->item_type;
 
+			update_item_number (self, e);
+			cf1->item_number --;
+			update_item_number (with, e);
+			cf1->item_number ++;
+
+			if (cf2_next_relative)
+				update_item_number (cf2_next_relative, e);
+		}
+	
+	}
+		
 	return rv;
 }
 
@@ -403,7 +439,44 @@ is_header (HTMLClueFlow *flow)
 	}
 }
 
-static guint get_post_padding (HTMLClueFlow *flow, guint pad);
+static gboolean
+need_blockquote_padding  (HTMLClueFlow *flow, HTMLClueFlow *prev)
+{
+	int i = get_similar_depth (flow, prev);
+	
+	/* 
+	 * If the levels don't match up the the current flow
+	 * level the padding should be handled the other way.
+	 */
+	if (i < flow->levels->len || flow->levels->len == 0) {
+		if (i < prev->levels->len)
+			return TRUE;
+                else 
+			return FALSE;
+	}
+	
+	i = prev->levels->len - i;
+	
+	/*
+	 * now we check each level greater than the current flow level
+	 * and see if it is a blockquote and therefore needs padding
+	 */
+	while (i > 0) {
+		HTMLListType type;
+		
+		type = prev->levels->data [prev->levels->len - i];
+
+		if (is_blockquote (type)) {
+			return TRUE;
+		}
+		i--;
+	}
+
+	/*
+	 * If all the levels were items we don't need padding
+	 */
+	return FALSE;
+}
 
 static guint
 get_pre_padding (HTMLClueFlow *flow, guint pad)
@@ -415,25 +488,18 @@ get_pre_padding (HTMLClueFlow *flow, guint pad)
 		return 0;
 
 	if (HTML_OBJECT_TYPE (prev_object) == HTML_TYPE_CLUEFLOW) {
-		HTMLClueFlow *prev;
+		HTMLClueFlow *prev = HTML_CLUEFLOW (prev_object);
 
-		if (get_post_padding (HTML_CLUEFLOW (prev_object), 1))
+		if (get_post_padding (prev, 1))
 			return 0;
-
-		if (is_item (HTML_CLUEFLOW (prev_object))) {
-			if (is_item (flow)) {
-				return 0;
-			} else {
+			
+		if (!is_levels_equal (flow, prev)) {
+			if (need_blockquote_padding (flow, prev))
 				return pad;
-			}
-		} else if (is_item (flow)) {
-			return pad;
+			else 
+				return 0;
 		}
 
-		prev = HTML_CLUEFLOW (prev_object);
-		if (prev->levels->len > flow->levels->len)
-			return pad;
-
 		if (flow->style == HTML_CLUEFLOW_STYLE_PRE
 		    && prev->style != HTML_CLUEFLOW_STYLE_PRE
 		    && ! is_header (prev))
@@ -447,7 +513,7 @@ get_pre_padding (HTMLClueFlow *flow, guint pad)
 
 	if (! is_header (flow) && flow->levels->len == 0)
 		return 0;
-
+	
 	return pad;
 }
 
@@ -464,13 +530,14 @@ get_post_padding (HTMLClueFlow *flow,
 	if (HTML_OBJECT_TYPE (next_object) == HTML_TYPE_CLUEFLOW) {
 		HTMLClueFlow *next;
 
-		if (is_item (flow) && is_item (HTML_CLUEFLOW (next_object)))
-			return 0;
-
 		next = HTML_CLUEFLOW (next_object);
-		if ((next->levels->len >= flow->levels->len)
-		    && !is_levels_equal (next, flow))
-			return pad;
+
+		if (!is_levels_equal (next, flow)) {
+			if (need_blockquote_padding (flow, next)) 
+				return pad;
+			else 
+				return 0;
+		}
 
 		if (flow->style == HTML_CLUEFLOW_STYLE_PRE
 		    && next->style != HTML_CLUEFLOW_STYLE_PRE
@@ -728,7 +795,7 @@ update_top_height (HTMLObject *begin, HTMLObject *end, gint *a, gint *d, gint *h
 static inline void
 update_line_positions (HTMLObject *clue, HTMLObject *begin, HTMLObject *end, gint left, gint a, gint d, gint height)
 {
-	gint xinc;
+	gint xinc = 0;
 
 	switch (html_clueflow_get_halignment (HTML_CLUEFLOW (clue))) {
 	case HTML_HALIGN_NONE:
@@ -802,7 +869,8 @@ layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 	x = start_lmargin = *lmargin;
 	o->ascent += o->y - old_y;
 
-	while (cur && !(cur->flags & HTML_OBJECT_FLAG_ALIGNED) && (x < *rmargin || first)) {
+	while (cur && !(cur->flags & HTML_OBJECT_FLAG_ALIGNED) && (HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE
+								   || (x < *rmargin || first))) {
 		HTMLFitType fit;
 		HTMLVAlignType valign;
 
@@ -821,7 +889,7 @@ layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 						  indent, &o->y, lmargin, rmargin);
 
 			/* is there enough space for this object? */
-			if (o->y != old_y && *rmargin - x < nb_width)
+			if (HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE && o->y != old_y && *rmargin - x < nb_width)
 				break;
 			need_update_height = TRUE;
 		}
@@ -1054,6 +1122,29 @@ get_item_number_str (HTMLClueFlow *flow)
 	}
 }
 
+static void
+draw_gt_line (HTMLObject *cur, HTMLPainter *p, gint offset, gint x, gint y)
+{
+	gint cy, a, d;
+
+	a = html_painter_calc_ascent (p, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
+	d = html_painter_calc_descent (p, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
+
+	cy = offset;
+	while (cy + a <= cur->ascent) {
+		html_painter_draw_text (p, x, y + cur->y - cy,
+					CLUEFLOW_BLOCKQUOTE_CITE, 1, 0);
+		cy += a + d;
+	}
+
+	cy = - offset + a + d;
+	while (cy + d <= cur->descent) {
+		html_painter_draw_text (p, x, y + cur->y + cy,
+					CLUEFLOW_BLOCKQUOTE_CITE, 1, 0);
+		cy += a + d;
+	}
+}
+		
 static void
 draw_quotes (HTMLObject *self, HTMLPainter *painter, 
 	     gint x, gint y, gint width, gint height,
@@ -1091,24 +1182,22 @@ draw_quotes (HTMLObject *self, HTMLPainter *painter,
 							paint.x0 + tx, paint.y0 + ty,
 							paint.x1 - paint.x0, paint.y1 - paint.y0);
 			} else {
-				/* draw "> " quote characters in the plain case */ 
 				HTMLObject *cur = HTML_CLUE (self)->head;
-				gint last_y = 0;
-			       
-
+				gint baseline = 0;
 				while (cur) {
-					if (cur->y != last_y) {
-						html_painter_set_font_style (painter, 
-								     html_clueflow_get_default_font_style (flow));
-						
-						html_painter_set_font_face  (painter, NULL);
-						html_painter_draw_text (painter, self->x + tx + last_indent,
-									self->y - self->ascent + cur->y + ty,
-									CLUEFLOW_BLOCKQUOTE_CITE, 1, 0);
+					if (cur->y != 0) {
+						baseline = cur->y;
+						break;
 					}
-					last_y = cur->y;
 					cur = cur->next;
 				}
+				/* draw "> " quote characters in the plain case */ 
+				html_painter_set_font_style (painter, 
+							     html_clueflow_get_default_font_style (flow));
+				
+				html_painter_set_font_face  (painter, NULL);
+				draw_gt_line (self, painter, self->ascent - baseline,
+					      self->x + tx + last_indent, ty);
 			}
 		}
 	}
@@ -1126,7 +1215,12 @@ draw_item (HTMLObject *self, HTMLPainter *painter, gint x, gint y, gint width, g
 		first = first->next;
 
 	flow = HTML_CLUEFLOW (self);
-	html_painter_set_pen (painter, &html_colorset_get_color_allocated (painter, HTMLTextColor)->color);
+
+	if (flow->item_color) {
+		html_color_alloc (flow->item_color, painter);
+		html_painter_set_pen (painter, &flow->item_color->color);
+	} else
+		html_painter_set_pen (painter, &html_colorset_get_color_allocated (painter, HTMLTextColor)->color);
 
 	indent = get_level_indent (flow, flow->levels->len - 1, painter);
 	if (flow->item_type == HTML_LIST_TYPE_UNORDERED) {
@@ -1227,6 +1321,12 @@ check_point (HTMLObject *self,
 
 	x = x - self->x;
 	y = y - self->y + self->ascent;
+	
+	/*
+	 * shift any selection inside the indent block to the 
+	 * left edge of the flow.
+	 */
+	x = MAX (x, get_indent (HTML_CLUEFLOW (self), painter));
 
 	for (p = clue->head; p; ) {
 
@@ -1304,15 +1404,6 @@ check_point (HTMLObject *self,
 	return NULL;
 }
 
-static void
-append_selection_string (HTMLObject *self,
-			 GString *buffer)
-{
-        (*HTML_OBJECT_CLASS (parent_class)->append_selection_string) (self, buffer);
-	if (self->selected)
-		g_string_append_c (buffer, '\n');
-}
-
 
 /* Saving support.  */
 
@@ -1537,7 +1628,10 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 	}
 
 	if (is_item (self)) {
-		if (!html_engine_save_output_string (state, "\n"))
+		if (next && is_levels_equal (self, next) && !is_item (next)) {
+			if (!html_engine_save_output_string (state, "<BR>\n"))
+				return FALSE;
+		} else if (!html_engine_save_output_string (state, "\n"))
 			return FALSE;
 	} else if (is_levels_equal (self, next) && self->style == next->style) {
 		if (self->style != HTML_CLUEFLOW_STYLE_PRE) {
@@ -1709,6 +1803,18 @@ plain_padding (HTMLClueFlow *flow, GString *out, gboolean firstline)
 	return pad_len;
 }
 
+static void
+append_selection_string (HTMLObject *self,
+			 GString *buffer)
+{
+        (*HTML_OBJECT_CLASS (parent_class)->append_selection_string) (self, buffer);
+
+	if (self->selected) {
+		g_string_append_c (buffer, '\n');
+		plain_padding (HTML_CLUEFLOW (self), buffer, TRUE);
+	}
+}
+
 static gboolean
 save_plain (HTMLObject *self,
 	    HTMLEngineSaveState *state,
@@ -1726,7 +1832,8 @@ save_plain (HTMLObject *self,
 	flow = HTML_CLUEFLOW (self);
 
 	pad = plain_padding (flow, NULL, FALSE);
-	buffer_state = html_engine_save_buffer_new (state->engine);
+	buffer_state = html_engine_save_buffer_new (state->engine, 
+						    state->inline_frames);
 
 	max_len = MAX (requested_width - pad, 0);
 	/* buffer the paragraph's content into the save buffer */
@@ -2171,6 +2278,7 @@ html_clueflow_init (HTMLClueFlow *clueflow, HTMLClueFlowClass *klass,
 
 	clueflow->item_type   = item_type;
 	clueflow->item_number = item_number;
+	clueflow->item_color = NULL;
 
 	clueflow->clear = clear;
 }
@@ -2331,8 +2439,14 @@ html_clueflow_set_levels (HTMLClueFlow *flow,
 			  HTMLEngine *engine,
 			  GByteArray *levels)
 {
+	HTMLObject *next_relative;
+
+	next_relative = get_next_relative_item (HTML_OBJECT (flow));
 	copy_levels (flow->levels, levels);
-	update_items_after_indentation_change (flow);
+
+	update_item_number (HTML_OBJECT (flow), engine);
+	if (next_relative)
+		update_item_number (next_relative, engine);
 
 	relayout_with_siblings (flow, engine);
 }
@@ -2353,9 +2467,9 @@ html_clueflow_set_item_type (HTMLClueFlow *flow,
 
 	flow->item_type = item_type;
 
-	update_item_number (HTML_OBJECT (flow));
+	update_item_number (HTML_OBJECT (flow), engine);
 	if (!items_are_relative (HTML_OBJECT (flow), HTML_OBJECT (flow)->next) && HTML_OBJECT (flow)->next)
-		update_item_number (HTML_OBJECT (flow)->next);
+		update_item_number (HTML_OBJECT (flow)->next, engine);
 
 	html_engine_schedule_update (engine);
 	/* FIXME - make it more effective: relayout_with_siblings (flow, engine); */
@@ -2408,28 +2522,20 @@ html_clueflow_get_halignment (HTMLClueFlow *flow)
 		return HTML_CLUE (flow)->halign;
 }
 
-static void
-update_items_after_indentation_change (HTMLClueFlow *flow)
-{
-	if (is_item (flow)) {
-		/* FIXME levels */
-		update_item_number (HTML_OBJECT (flow));
-		if (HTML_OBJECT (flow)->next && is_item (HTML_CLUEFLOW (HTML_OBJECT (flow)->next)))
-			update_item_number (HTML_OBJECT (flow)->next);
-	}
-}
-
 void
 html_clueflow_modify_indentation_by_delta (HTMLClueFlow *flow,
 					   HTMLEngine *engine,
 					   gint indentation_delta,
 					   guint8 *indentation_levels)
 {
+	HTMLObject *next_relative;
 	gint indentation;
 	g_return_if_fail (flow != NULL);
 	g_return_if_fail (engine != NULL);
 	g_return_if_fail (HTML_IS_ENGINE (engine));
 
+	next_relative = get_next_relative_item (HTML_OBJECT (flow));
+
 	indentation = flow->levels->len + indentation_delta;
 	indentation = indentation < 0 ? 0 : indentation;
 
@@ -2444,7 +2550,9 @@ html_clueflow_modify_indentation_by_delta (HTMLClueFlow *flow,
 		}
 	}
 
-	update_items_after_indentation_change (flow);
+	update_item_number (HTML_OBJECT (flow), engine);
+	if (next_relative)
+		update_item_number (next_relative, engine);
 	relayout_with_siblings (flow, engine);
 }
 
@@ -2454,6 +2562,7 @@ html_clueflow_set_indentation (HTMLClueFlow *flow,
 			       gint indentation,
 			       guint8 *indentation_levels)
 {
+	HTMLObject *next_relative;
 	int i;
 	g_return_if_fail (flow != NULL);
 	g_return_if_fail (engine != NULL);
@@ -2462,13 +2571,17 @@ html_clueflow_set_indentation (HTMLClueFlow *flow,
 	if (indentation < 0)
 		indentation = 0;
 
+	next_relative = get_next_relative_item (HTML_OBJECT (flow));
+
 	g_byte_array_set_size (flow->levels, indentation);
 
 	i = indentation;
 	while (i--)
 		flow->levels->data[i] = indentation_levels[i];
 
-	update_items_after_indentation_change (flow);
+	update_item_number (HTML_OBJECT (flow), engine);
+	if (next_relative)
+		update_item_number (next_relative, engine);
 	relayout_with_siblings (flow, engine);
 }
 
@@ -2519,27 +2632,6 @@ html_clueflow_get_properties (HTMLClueFlow *flow,
 }
 #endif
 
-
-void
-html_clueflow_remove_text_slaves (HTMLClueFlow *flow)
-{
-	HTMLClue *clue;
-	HTMLObject *p;
-	HTMLObject *pnext;
-
-	g_return_if_fail (flow != NULL);
-
-	clue = HTML_CLUE (flow);
-	for (p = clue->head; p != NULL; p = pnext) {
-		pnext = p->next;
-
-		if (HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE) {
-			html_clue_remove (clue, p);
-			html_object_destroy (p);
-		}
-	}
-}
-
 /* spell checking */
 
 #include "htmlinterval.h"
@@ -2826,3 +2918,25 @@ html_clueflow_tabs (HTMLClueFlow *flow, HTMLPainter *p)
 	return (flow && HTML_IS_CLUEFLOW (flow) && flow->style == HTML_CLUEFLOW_STYLE_PRE) || HTML_IS_PLAIN_PAINTER (p)
 		? TRUE : FALSE;
 }
+
+void
+html_clueflow_set_item_color (HTMLClueFlow *flow, HTMLColor *color)
+{
+	if (flow->item_color)
+		html_color_unref (flow->item_color);
+	if (color)
+		html_color_ref (color);
+	flow->item_color = color;
+}
+
+gboolean
+html_clueflow_style_equals (HTMLClueFlow *cf1, HTMLClueFlow *cf2)
+{
+	if (!cf1 || !cf2
+	    || !HTML_IS_CLUEFLOW (cf1) || !HTML_IS_CLUEFLOW (cf2)
+	    || cf1->style != cf2->style
+	    || (cf1->style == HTML_CLUEFLOW_STYLE_LIST_ITEM && cf1->item_type != cf2->item_type)
+	    || !is_levels_equal (cf1, cf2))
+		return FALSE;
+	return TRUE;
+}
